@@ -2,7 +2,7 @@
 // GB_bitmap_assign_M_noaccum_whole:  assign to C bitmap
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2023, All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
@@ -22,8 +22,12 @@
 // A:           matrix (hyper, sparse, bitmap, or full), or scalar
 // kind:        assign or subassign (same action)
 
-#include "GB_bitmap_assign_methods.h"
+// JIT: needed.
 
+#include "GB_bitmap_assign_methods.h"
+#include "GB_assign_shared_definitions.h"
+
+#undef  GB_FREE_ALL
 #define GB_FREE_ALL                         \
 {                                           \
     GB_WERK_POP (M_ek_slicing, int64_t) ;   \
@@ -42,7 +46,7 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
     const GrB_Matrix A,         // input matrix, not transposed
     const void *scalar,         // input scalar
     const GrB_Type scalar_type, // type of input scalar
-    GB_Context Context
+    GB_Werk Werk
 )
 {
 
@@ -87,7 +91,7 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
             // Cb [pC] += 2 for each entry M(i,j) in the mask
             GB_bitmap_M_scatter_whole (C,
                 M, Mask_struct, GB_BITMAP_M_SCATTER_PLUS_2,
-                M_ek_slicing, M_ntasks, M_nthreads, Context) ;
+                M_ek_slicing, M_ntasks, M_nthreads) ;
             // the bitmap of C now contains:
             //  Cb (i,j) = 0:   cij not present, mij zero
             //  Cb (i,j) = 1:   cij present, mij zero
@@ -109,13 +113,13 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                         break ;                                     \
                     case 2: /* C(i,j) not present, M(i,j) = 1 */    \
                         /* Cx [pC] = scalar */                      \
-                        GB_ASSIGN_SCALAR (pC) ;                     \
+                        GB_COPY_scalar_to_C (Cx, pC, cwork) ;       \
                         Cb [pC] = 1 ;                               \
                         task_cnvals++ ;                             \
                         break ;                                     \
                     case 3:  /* C(i,j) present, M(i,j) = 1 */       \
                         /* Cx [pC] = scalar */                      \
-                        GB_ASSIGN_SCALAR (pC) ;                     \
+                        GB_COPY_scalar_to_C (Cx, pC, cwork) ;       \
                         Cb [pC] = 1 ;                               \
                     default: ;                                      \
                 }                                                   \
@@ -137,13 +141,13 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 {                                           \
                     /* C(i,j) present, M(i,j) = 1 */        \
                     /* Cx [pC] = scalar */                  \
-                    GB_ASSIGN_SCALAR (pC) ;                 \
+                    GB_COPY_scalar_to_C (Cx, pC, cwork) ;   \
                 }                                           \
                 else                                        \
                 {                                           \
                     /* C(i,j) not present, M(i,j) = 1 */    \
                     /* Cx [pC] = scalar */                  \
-                    GB_ASSIGN_SCALAR (pC) ;                 \
+                    GB_COPY_scalar_to_C (Cx, pC, cwork) ;   \
                     Cb [pC] = 1 ;                           \
                     task_cnvals++ ;                         \
                 }                                           \
@@ -177,7 +181,7 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 // Cb [pC] += 2 for each entry M(i,j) in the mask
                 GB_bitmap_M_scatter_whole (C,
                     M, Mask_struct, GB_BITMAP_M_SCATTER_PLUS_2,
-                    M_ek_slicing, M_ntasks, M_nthreads, Context) ;
+                    M_ek_slicing, M_ntasks, M_nthreads) ;
                 // the bitmap of C now contains:
                 //  Cb (i,j) = 0:   cij not present, mij zero
                 //  Cb (i,j) = 1:   cij present, mij zero
@@ -185,44 +189,44 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 //  Cb (i,j) = 3:   cij present, mij 1
 
                 #undef  GB_CIJ_WORK
-                #define GB_CIJ_WORK(pC)                                 \
-                {                                                       \
-                    switch (Cb [pC])                                    \
-                    {                                                   \
-                        case 1: /* C(i,j) present, M(i,j) = 0 */        \
-                            Cb [pC] = 0 ;                               \
-                            task_cnvals-- ;                             \
-                            break ;                                     \
-                        case 2: /* C(i,j) not present, M(i,j) = 1 */    \
-                            if (GBB (Ab, pC))                           \
-                            {                                           \
-                                /* Cx [pC] = Ax [pC] */                 \
-                                GB_ASSIGN_AIJ (pC, pC) ;                \
-                                Cb [pC] = 1 ;                           \
-                                task_cnvals++ ;                         \
-                            }                                           \
-                            else                                        \
-                            {                                           \
-                                /* clear the mask from C */             \
-                                Cb [pC] = 0 ;                           \
-                            }                                           \
-                            break ;                                     \
-                        case 3: /* C(i,j) present, M(i,j) = 1 */        \
-                            if (GBB (Ab, pC))                           \
-                            {                                           \
-                                /* Cx [pC] = Ax [pC] */                 \
-                                GB_ASSIGN_AIJ (pC, pC) ;                \
-                                Cb [pC] = 1 ;                           \
-                            }                                           \
-                            else                                        \
-                            {                                           \
-                                /* delete C(i,j) */                     \
-                                Cb [pC] = 0 ;                           \
-                                task_cnvals-- ;                         \
-                            }                                           \
-                            break ;                                     \
-                        default: ;                                      \
-                    }                                                   \
+                #define GB_CIJ_WORK(pC)                                       \
+                {                                                             \
+                    switch (Cb [pC])                                          \
+                    {                                                         \
+                        case 1: /* C(i,j) present, M(i,j) = 0 */              \
+                            Cb [pC] = 0 ;                                     \
+                            task_cnvals-- ;                                   \
+                            break ;                                           \
+                        case 2: /* C(i,j) not present, M(i,j) = 1 */          \
+                            if (GBB (Ab, pC))                                 \
+                            {                                                 \
+                                /* Cx [pC] = Ax [pC] */                       \
+                                GB_COPY_aij_to_C (Cx,pC,Ax,pC,A_iso,cwork) ;  \
+                                Cb [pC] = 1 ;                                 \
+                                task_cnvals++ ;                               \
+                            }                                                 \
+                            else                                              \
+                            {                                                 \
+                                /* clear the mask from C */                   \
+                                Cb [pC] = 0 ;                                 \
+                            }                                                 \
+                            break ;                                           \
+                        case 3: /* C(i,j) present, M(i,j) = 1 */              \
+                            if (GBB (Ab, pC))                                 \
+                            {                                                 \
+                                /* Cx [pC] = Ax [pC] */                       \
+                                GB_COPY_aij_to_C (Cx,pC,Ax,pC,A_iso,cwork) ;  \
+                                Cb [pC] = 1 ;                                 \
+                            }                                                 \
+                            else                                              \
+                            {                                                 \
+                                /* delete C(i,j) */                           \
+                                Cb [pC] = 0 ;                                 \
+                                task_cnvals-- ;                               \
+                            }                                                 \
+                            break ;                                           \
+                        default: ;                                            \
+                    }                                                         \
                 }
                 #include "GB_bitmap_assign_C_whole_template.c"
 
@@ -235,34 +239,34 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 //--------------------------------------------------------------
 
                 #undef  GB_MASK_WORK
-                #define GB_MASK_WORK(pC)                        \
-                {                                               \
-                    if (Cb [pC])                                \
-                    {                                           \
-                        /* C(i,j) present, M(i,j) = 1 */        \
-                        if (GBB (Ab, pC))                       \
-                        {                                       \
-                            /* Cx [pC] = Ax [pC] */             \
-                            GB_ASSIGN_AIJ (pC, pC) ;            \
-                        }                                       \
-                        else                                    \
-                        {                                       \
-                            /* delete C(i,j) */                 \
-                            Cb [pC] = 0 ;                       \
-                            task_cnvals-- ;                     \
-                        }                                       \
-                    }                                           \
-                    else                                        \
-                    {                                           \
-                        /* C(i,j) not present, M(i,j) = 1 */    \
-                        if (GBB (Ab, pC))                       \
-                        {                                       \
-                            /* Cx [pC] = Ax [pC] */             \
-                            GB_ASSIGN_AIJ (pC, pC) ;            \
-                            Cb [pC] = 1 ;                       \
-                            task_cnvals++ ;                     \
-                        }                                       \
-                    }                                           \
+                #define GB_MASK_WORK(pC)                                      \
+                {                                                             \
+                    if (Cb [pC])                                              \
+                    {                                                         \
+                        /* C(i,j) present, M(i,j) = 1 */                      \
+                        if (GBB (Ab, pC))                                     \
+                        {                                                     \
+                            /* Cx [pC] = Ax [pC] */                           \
+                            GB_COPY_aij_to_C (Cx, pC, Ax, pC, A_iso, cwork) ; \
+                        }                                                     \
+                        else                                                  \
+                        {                                                     \
+                            /* delete C(i,j) */                               \
+                            Cb [pC] = 0 ;                                     \
+                            task_cnvals-- ;                                   \
+                        }                                                     \
+                    }                                                         \
+                    else                                                      \
+                    {                                                         \
+                        /* C(i,j) not present, M(i,j) = 1 */                  \
+                        if (GBB (Ab, pC))                                     \
+                        {                                                     \
+                            /* Cx [pC] = Ax [pC] */                           \
+                            GB_COPY_aij_to_C (Cx, pC, Ax, pC, A_iso, cwork) ; \
+                            Cb [pC] = 1 ;                                     \
+                            task_cnvals++ ;                                   \
+                        }                                                     \
+                    }                                                         \
                 }
                 #include "GB_bitmap_assign_M_all_template.c"
 
@@ -285,7 +289,7 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 // Cb [pC] += 2 for each entry M(i,j) in the mask
                 GB_bitmap_M_scatter_whole (C,
                     M, Mask_struct, GB_BITMAP_M_SCATTER_PLUS_2,
-                    M_ek_slicing, M_ntasks, M_nthreads, Context) ;
+                    M_ek_slicing, M_ntasks, M_nthreads) ;
                 // the bitmap of C now contains:
                 //  Cb (i,j) = 0:   cij not present, mij zero
                 //  Cb (i,j) = 1:   cij present, mij zero
@@ -294,17 +298,17 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
 
                 // assign entries from A into C
                 #undef  GB_AIJ_WORK
-                #define GB_AIJ_WORK(pC,pA)                      \
-                {                                               \
-                    int8_t cb = Cb [pC] ;                       \
-                    if (cb >= 2)                                \
-                    {                                           \
-                        /* M(i,j)=1 and A(i,j) present */       \
-                        /* Cx [pC] = Ax [pA] ; */               \
-                        GB_ASSIGN_AIJ (pC, pA) ;                \
-                        Cb [pC] = 4 ;                           \
-                        task_cnvals += (cb == 2) ;              \
-                    }                                           \
+                #define GB_AIJ_WORK(pC,pA)                                  \
+                {                                                           \
+                    int8_t cb = Cb [pC] ;                                   \
+                    if (cb >= 2)                                            \
+                    {                                                       \
+                        /* M(i,j)=1 and A(i,j) present */                   \
+                        /* Cx [pC] = Ax [pA] ; */                           \
+                        GB_COPY_aij_to_C (Cx, pC, Ax, pA, A_iso, cwork) ;   \
+                        Cb [pC] = 4 ;                                       \
+                        task_cnvals += (cb == 2) ;                          \
+                    }                                                       \
                 }
                 #include "GB_bitmap_assign_A_whole_template.c"
 
@@ -319,8 +323,8 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 #include "GB_bitmap_assign_C_whole_template.c"
 
             }
-            else if (GB_NNZ (A) == 0)
-            {
+            else if (GB_nnz (A) == 0)
+            { 
 
                 //--------------------------------------------------------------
                 // C<M> = A where A is sparse or hyper, with no entries
@@ -349,7 +353,7 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
                 // Cb [pC] += 2 for each entry M(i,j) in the mask
                 GB_bitmap_M_scatter_whole (C,
                     M, Mask_struct, GB_BITMAP_M_SCATTER_PLUS_2,
-                    M_ek_slicing, M_ntasks, M_nthreads, Context) ;
+                    M_ek_slicing, M_ntasks, M_nthreads) ;
                 // the bitmap of C now contains:
                 //  Cb (i,j) = 0:   cij not present, mij zero
                 //  Cb (i,j) = 1:   cij present, mij zero
@@ -358,17 +362,17 @@ GrB_Info GB_bitmap_assign_M_noaccum_whole
 
                 // assign entries from A into C
                 #undef  GB_AIJ_WORK
-                #define GB_AIJ_WORK(pC,pA)                      \
-                {                                               \
-                    int8_t cb = Cb [pC] ;                       \
-                    if (cb >= 2)                                \
-                    {                                           \
-                        /* M(i,j)=1 and A(i,j) present */       \
-                        /* Cx [pC] = Ax [pA] ; */               \
-                        GB_ASSIGN_AIJ (pC, pA) ;                \
-                        Cb [pC] = 1 ;                           \
-                        task_cnvals += (cb == 2) ;              \
-                    }                                           \
+                #define GB_AIJ_WORK(pC,pA)                                  \
+                {                                                           \
+                    int8_t cb = Cb [pC] ;                                   \
+                    if (cb >= 2)                                            \
+                    {                                                       \
+                        /* M(i,j)=1 and A(i,j) present */                   \
+                        /* Cx [pC] = Ax [pA] ; */                           \
+                        GB_COPY_aij_to_C (Cx, pC, Ax, pA, A_iso, cwork) ;   \
+                        Cb [pC] = 1 ;                                       \
+                        task_cnvals += (cb == 2) ;                          \
+                    }                                                       \
                 }
                 #include "GB_bitmap_assign_A_whole_template.c"
 
