@@ -2,7 +2,7 @@
 // GrB_Matrix_setElement: set an entry in a matrix, C(row,col) = x
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2022, All Rights Reserved.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2023, All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
@@ -10,6 +10,7 @@
 // Set a single entry in a matrix, C(row,col) = x,
 // typecasting from the type of x to the type of C, as needed.
 
+#define GB_FREE_ALL ;
 #include "GB.h"
 
 #define GB_SET(prefix,type,T,ampersand)                                     \
@@ -25,7 +26,7 @@ GrB_Info GB_EVAL3 (prefix, _Matrix_setElement_, T) /* C (row,col) = x */    \
         " (C, row, col, x)") ;                                              \
     GB_RETURN_IF_NULL_OR_FAULTY (C) ;                                       \
     return (GB_setElement (C, NULL, ampersand x, row, col,                  \
-        GB_ ## T ## _code, Context)) ;                                      \
+        GB_ ## T ## _code, Werk)) ;                                      \
 }
 
 GB_SET (GrB, bool      , BOOL   , &)
@@ -72,16 +73,17 @@ GrB_Info GrB_Matrix_setElement_Scalar
     // set or remove the element
     //--------------------------------------------------------------------------
 
+    GB_MATRIX_WAIT (scalar) ;
     if (GB_nnz ((GrB_Matrix) scalar) > 0)
     { 
         // set the element: C(row,col) = scalar
         return (GB_setElement (C, NULL, scalar->x, row, col,
-            scalar->type->code, Context)) ;
+            scalar->type->code, Werk)) ;
     }
     else
     { 
         // delete the C(row,col) element
-        return (GB_Matrix_removeElement (C, row, col, Context)) ;
+        return (GB_Matrix_removeElement (C, row, col, Werk)) ;
     }
 }
 
