@@ -2,6 +2,9 @@
 // === spqr.hpp ================================================================
 // =============================================================================
 
+// SPQR, Copyright (c) 2008-2022, Timothy A Davis. All Rights Reserved.
+// SPDX-License-Identifier: GPL-2.0+
+
 // Internal definitions and non-user-callable routines.  This should not be
 // included in the user's code.
 
@@ -12,6 +15,7 @@
 // include files
 // -----------------------------------------------------------------------------
 
+#define SUITESPARSE_BLAS_DEFINITIONS
 #include "SuiteSparseQR.hpp"
 #include <stdlib.h>
 #include <math.h>
@@ -51,12 +55,6 @@ typedef std::complex<double> Complex ;
 #define DEBUG_EXPENSIVE
 */
 
-// -----------------------------------------------------------------------------
-// Long is defined at SuiteSparse_long, from SuiteSparse_config.h
-// -----------------------------------------------------------------------------
-
-#define Long SuiteSparse_long
-
 // -----------------------------------------------------------------------------
 // basic macros
 // -----------------------------------------------------------------------------
@@ -94,7 +92,7 @@ typedef std::complex<double> Complex ;
 
 #define ITYPE CHOLMOD_LONG
 #define DTYPE CHOLMOD_DOUBLE
-#define ID SuiteSparse_long_id
+#define ID "%" PRId64
 
 // -----------------------------------------------------------------------------
 
@@ -211,16 +209,16 @@ typedef std::complex<double> Complex ;
 // workspace required for each stack in spqr_factorize and spqr_kernel
 template <typename Entry> struct spqr_work
 {
-    Long *Stair1 ;          // size maxfn if H not kept
-    Long *Cmap ;            // size maxfn
-    Long *Fmap ;            // size n
+    int64_t *Stair1 ;          // size maxfn if H not kept
+    int64_t *Cmap ;            // size maxfn
+    int64_t *Fmap ;            // size n
     Entry *WTwork ;         // size (fchunk + (keepH ? 0:1)) * maxfn
 
     Entry *Stack_head ;     // head of Stack
     Entry *Stack_top ;      // top of Stack
 
-    Long sumfrank ;         // sum of ranks of the fronts in this stack
-    Long maxfrank ;         // largest rank of fronts in this stack
+    int64_t sumfrank ;         // sum of ranks of the fronts in this stack
+    int64_t maxfrank ;         // largest rank of fronts in this stack
 
     // for computing the 2-norm of w, the vector of the dead column norms
     double wscale ;         // scale factor for norm (w (of this stack))
@@ -240,11 +238,11 @@ template <typename Entry> struct spqr_blob
     spqr_symbolic *QRsym ;
     spqr_numeric <Entry> *QRnum ;
     spqr_work <Entry> *Work ;
-    Long *Cm ;
+    int64_t *Cm ;
     Entry **Cblock ;
     Entry *Sx ;
-    Long ntol ;
-    Long fchunk ;
+    int64_t ntol ;
+    int64_t fchunk ;
     cholmod_common *cc ;
 } ;
 
@@ -258,7 +256,7 @@ spqr_symbolic *spqr_analyze
     // inputs, not modified
     cholmod_sparse *A,
     int ordering,           // all ordering options available
-    Long *Quser,            // user provided ordering, if given (may be NULL)
+    int64_t *Quser,            // user provided ordering, if given (may be NULL)
 
     int do_rank_detection,  // if TRUE, then rank deficient matrices may be
                             // considered during numerical factorization,
@@ -278,9 +276,9 @@ template <typename Entry> spqr_numeric <Entry> *spqr_factorize
     cholmod_sparse **Ahandle,
 
     // inputs, not modified
-    Long freeA,                     // if TRUE, free A on output
+    int64_t freeA,                     // if TRUE, free A on output
     double tol,                     // for rank detection
-    Long ntol,                      // apply tol only to first ntol columns
+    int64_t ntol,                      // apply tol only to first ntol columns
     spqr_symbolic *QRsym,
 
     // workspace and parameters
@@ -308,13 +306,13 @@ template <typename Entry> double spqr_maxcolnorm
 
 template <typename Entry> void spqr_kernel
 (
-    Long task,
+    int64_t task,
     spqr_blob <Entry> *Blob
 ) ;
 
 template <typename Entry> void spqr_parallel
 (
-    Long ntasks,
+    int64_t ntasks,
     int nthreads,
     spqr_blob <Entry> *Blob
 ) ;
@@ -347,16 +345,16 @@ void spqr_stranspose1
 (
     // input, not modified
     cholmod_sparse *A,  // m-by-n
-    Long *Qfill,        // size n, fill-reducing column permutation;
+    int64_t *Qfill,        // size n, fill-reducing column permutation;
                         // Qfill [k] = j if the kth column of S is the jth
                         // column of A.  Identity permutation is used if
                         // Qfill is NULL.
 
     // output, contents not defined on input
-    Long *Sp,           // size m+1, row pointers of S
-    Long *Sj,           // size nz, column indices of S
-    Long *PLinv,        // size m, inverse row permutation, PLinv [i] = k
-    Long *Sleft,        // size n+2, Sleft [j] ... Sleft [j+1]-1 is the list of
+    int64_t *Sp,           // size m+1, row pointers of S
+    int64_t *Sj,           // size nz, column indices of S
+    int64_t *PLinv,        // size m, inverse row permutation, PLinv [i] = k
+    int64_t *Sleft,        // size n+2, Sleft [j] ... Sleft [j+1]-1 is the list of
                         // rows of S whose leftmost column index is j.  The list
                         // can be empty (that is, Sleft [j] == Sleft [j+1]).
                         // Sleft [n] is the number of non-empty rows of S, and
@@ -364,7 +362,7 @@ void spqr_stranspose1
                         // Sleft [n+1]-1 gives the empty rows of S.
 
     // workspace, not defined on input or output
-    Long *W             // size m
+    int64_t *W             // size m
 ) ;
 
 
@@ -372,19 +370,19 @@ template <typename Entry> void spqr_stranspose2
 (
     // input, not modified
     cholmod_sparse *A,  // m-by-n
-    Long *Qfill,        // size n, fill-reducing column permutation;
+    int64_t *Qfill,        // size n, fill-reducing column permutation;
                         // Qfill [k] = j
                         // if the kth column of S is the jth column of A.
                         // Identity permutation is used if Qfill is NULL.
 
-    Long *Sp,           // size m+1, row pointers of S
-    Long *PLinv,        // size m, inverse row permutation, PLinv [i] = k
+    int64_t *Sp,           // size m+1, row pointers of S
+    int64_t *PLinv,        // size m, inverse row permutation, PLinv [i] = k
 
     // output, contents not defined on input
     Entry *Sx,          // size nz, numerical values of S
 
     // workspace, not defined on input or output
-    Long *W             // size m
+    int64_t *W             // size m
 ) ;
 
 
@@ -395,110 +393,110 @@ template <typename Entry> void spqr_stranspose2
 template <typename Entry> void spqrDebug_dumpdense
 (
     Entry *A,
-    Long m,
-    Long n,
-    Long lda,
+    int64_t m,
+    int64_t n,
+    int64_t lda,
     cholmod_common *cc
 ) ;
 
 template <typename Entry> void spqrDebug_dumpsparse
 (
-    Long *Ap,
-    Long *Ai,
+    int64_t *Ap,
+    int64_t *Ai,
     Entry *Ax,
-    Long m,
-    Long n,
+    int64_t m,
+    int64_t n,
     cholmod_common *cc
 ) ;
 
 void spqrDebug_print (double x) ;
 void spqrDebug_print (Complex x) ;
 
-void spqrDebug_dump_Parent (Long n, Long *Parent, const char *filename) ;
+void spqrDebug_dump_Parent (int64_t n, int64_t *Parent, const char *filename) ;
 
-Long spqrDebug_rhsize             // returns # of entries in R+H block
+int64_t spqrDebug_rhsize             // returns # of entries in R+H block
 (
     // input, not modified
-    Long m,                 // # of rows in F
-    Long n,                 // # of columns in F
-    Long npiv,              // number of pivotal columns in F
-    Long *Stair,            // size n; column j is dead if Stair [j] == 0.
+    int64_t m,                 // # of rows in F
+    int64_t n,                 // # of columns in F
+    int64_t npiv,              // number of pivotal columns in F
+    int64_t *Stair,            // size n; column j is dead if Stair [j] == 0.
                             // Only the first npiv columns can be dead.
     cholmod_common *cc
 ) ;
 #endif
 
 #ifdef DEBUG_EXPENSIVE
-Long spqrDebug_listcount
+int64_t spqrDebug_listcount
 (
-    Long x, Long *List, Long len, Long what,
+    int64_t x, int64_t *List, int64_t len, int64_t what,
     cholmod_common *cc
 ) ;
 #endif
 
 // =============================================================================
 
-Long spqr_fsize     // returns # of rows of F
+int64_t spqr_fsize     // returns # of rows of F
 (
     // inputs, not modified
-    Long f,
-    Long *Super,            // size nf, from QRsym
-    Long *Rp,               // size nf, from QRsym
-    Long *Rj,               // size rjsize, from QRsym
-    Long *Sleft,            // size n+2, from QRsym
-    Long *Child,            // size nf, from QRsym
-    Long *Childp,           // size nf+1, from QRsym
-    Long *Cm,               // size nf, from QRwork
+    int64_t f,
+    int64_t *Super,            // size nf, from QRsym
+    int64_t *Rp,               // size nf, from QRsym
+    int64_t *Rj,               // size rjsize, from QRsym
+    int64_t *Sleft,            // size n+2, from QRsym
+    int64_t *Child,            // size nf, from QRsym
+    int64_t *Childp,           // size nf+1, from QRsym
+    int64_t *Cm,               // size nf, from QRwork
 
     // outputs, not defined on input
-    Long *Fmap,             // size n, from QRwork
-    Long *Stair             // size fn, from QRwork
+    int64_t *Fmap,             // size n, from QRwork
+    int64_t *Stair             // size fn, from QRwork
 ) ;
 
 
 template <typename Entry> void spqr_assemble
 (
     // inputs, not modified
-    Long f,                 // front to assemble F
-    Long fm,                // number of rows of F
+    int64_t f,                 // front to assemble F
+    int64_t fm,                // number of rows of F
     int keepH,              // if TRUE, then construct row pattern of H
-    Long *Super,
-    Long *Rp,
-    Long *Rj,
-    Long *Sp,
-    Long *Sj,
-    Long *Sleft,
-    Long *Child,
-    Long *Childp,
+    int64_t *Super,
+    int64_t *Rp,
+    int64_t *Rj,
+    int64_t *Sp,
+    int64_t *Sj,
+    int64_t *Sleft,
+    int64_t *Child,
+    int64_t *Childp,
     Entry *Sx,
-    Long *Fmap,
-    Long *Cm,
+    int64_t *Fmap,
+    int64_t *Cm,
     Entry **Cblock,
 #ifndef NDEBUG
     char *Rdead,
 #endif
-    Long *Hr,
+    int64_t *Hr,
 
     // input/output
-    Long *Stair,
-    Long *Hii,              // if keepH, construct list of row indices for F
+    int64_t *Stair,
+    int64_t *Hii,              // if keepH, construct list of row indices for F
     // input only
-    Long *Hip,
+    int64_t *Hip,
 
     // outputs, not defined on input
     Entry *F,
 
     // workspace, not defined on input or output
-    Long *Cmap
+    int64_t *Cmap
 ) ;
 
-template <typename Entry> Long spqr_cpack     // returns # of rows in C
+template <typename Entry> int64_t spqr_cpack     // returns # of rows in C
 (
     // input, not modified
-    Long m,                 // # of rows in F
-    Long n,                 // # of columns in F
-    Long npiv,              // number of pivotal columns in F
-    Long g,                 // the C block starts at F (g,npiv)
+    int64_t m,                 // # of rows in F
+    int64_t n,                 // # of columns in F
+    int64_t npiv,              // number of pivotal columns in F
+    int64_t g,                 // the C block starts at F (g,npiv)
 
     // input, not modified unless the pack occurs in-place
     Entry *F,               // m-by-n frontal matrix in column-major order
@@ -508,22 +506,22 @@ template <typename Entry> Long spqr_cpack     // returns # of rows in C
                             // trapezoidal form.
 ) ;
 
-Long spqr_fcsize    // returns # of entries in C of current front F
+int64_t spqr_fcsize    // returns # of entries in C of current front F
 (
     // input, not modified
-    Long m,                 // # of rows in F
-    Long n,                 // # of columns in F
-    Long npiv,              // number of pivotal columns in F
-    Long g                  // the C block starts at F (g,npiv)
+    int64_t m,                 // # of rows in F
+    int64_t n,                 // # of columns in F
+    int64_t npiv,              // number of pivotal columns in F
+    int64_t g                  // the C block starts at F (g,npiv)
 ) ;
 
-Long spqr_csize     // returns # of entries in C of a child
+int64_t spqr_csize     // returns # of entries in C of a child
 (
     // input, not modified
-    Long c,                 // child c
-    Long *Rp,               // size nf+1, pointers for pattern of R
-    Long *Cm,               // size nf, Cm [c] = # of rows in child C
-    Long *Super             // size nf, pivotal columns in each front
+    int64_t c,                 // child c
+    int64_t *Rp,               // size nf+1, pointers for pattern of R
+    int64_t *Cm,               // size nf, Cm [c] = # of rows in child C
+    int64_t *Super             // size nf, pivotal columns in each front
 ) ;
 
 template <typename Entry> void spqr_rcount
@@ -532,21 +530,21 @@ template <typename Entry> void spqr_rcount
     spqr_symbolic *QRsym,
     spqr_numeric <Entry> *QRnum,
 
-    Long n1rows,        // added to each row index of Ra and Rb
-    Long econ,          // only get entries in rows n1rows to econ-1
-    Long n2,            // Ra = R (:,0:n2-1), Rb = R (:,n2:n-1)
+    int64_t n1rows,        // added to each row index of Ra and Rb
+    int64_t econ,          // only get entries in rows n1rows to econ-1
+    int64_t n2,            // Ra = R (:,0:n2-1), Rb = R (:,n2:n-1)
     int getT,           // if true, count Rb' instead of Rb
 
     // input/output
-    Long *Ra,           // size n2; Ra [j] += nnz (R (:,j)) if j < n2
-    Long *Rb,           // If getT is false: size n-n2 and
+    int64_t *Ra,           // size n2; Ra [j] += nnz (R (:,j)) if j < n2
+    int64_t *Rb,           // If getT is false: size n-n2 and
                         // Rb [j-n2] += nnz (R (:,j)) if j >= n2.
                         // If getT is true: size econ, and
                         // Rb [i] += nnz (R (i, n2:n-1))
-    Long *Hp,           // size rjsize+1.  Column pointers for H.
+    int64_t *Hp,           // size rjsize+1.  Column pointers for H.
                         // Only computed if H was kept during factorization.
                         // Only Hp [0..nh] is used.
-    Long *p_nh          // number of Householder vectors (nh <= rjsize)
+    int64_t *p_nh          // number of Householder vectors (nh <= rjsize)
 ) ;
 
 template <typename Entry> void spqr_rconvert
@@ -555,22 +553,22 @@ template <typename Entry> void spqr_rconvert
     spqr_symbolic *QRsym,
     spqr_numeric <Entry> *QRnum,
 
-    Long n1rows,        // added to each row index of Ra, Rb, and H
-    Long econ,          // only get entries in rows n1rows to econ-1
-    Long n2,            // Ra = R (:,0:n2-1), Rb = R (:,n2:n-1)
+    int64_t n1rows,        // added to each row index of Ra, Rb, and H
+    int64_t econ,          // only get entries in rows n1rows to econ-1
+    int64_t n2,            // Ra = R (:,0:n2-1), Rb = R (:,n2:n-1)
     int getT,           // if true, get Rb' instead of Rb
 
     // input/output
-    Long *Rap,          // size n2+1; on input, Rap [j] is the column pointer
+    int64_t *Rap,          // size n2+1; on input, Rap [j] is the column pointer
                         // for Ra.  Incremented on output by the number of
                         // entries added to column j of Ra.
 
     // output, not defined on input
-    Long *Rai,          // size rnz1 = nnz(Ra); row indices of Ra
+    int64_t *Rai,          // size rnz1 = nnz(Ra); row indices of Ra
     Entry *Rax,         // size rnz; numerical values of Ra
 
     // input/output
-    Long *Rbp,          // if getT is false:
+    int64_t *Rbp,          // if getT is false:
                         // size (n-n2)+1; on input, Rbp [j] is the column
                         // pointer for Rb.  Incremented on output by the number
                         // of entries added to column j of Rb.
@@ -580,28 +578,28 @@ template <typename Entry> void spqr_rconvert
                         // of entries added to row i of Rb.
 
     // output, not defined on input
-    Long *Rbi,          // size rnz2 = nnz(Rb); indices of Rb
+    int64_t *Rbi,          // size rnz2 = nnz(Rb); indices of Rb
     Entry *Rbx,         // size rnz2; numerical values of Rb
 
     // input
-    Long *H2p,          // size nh+1; H2p [j] is the column pointer for H.
+    int64_t *H2p,          // size nh+1; H2p [j] is the column pointer for H.
                         // H2p, H2i, and H2x are ignored if H was not kept
                         // during factorization.  nh computed by rcount
 
     // output, not defined on input
-    Long *H2i,           // size hnz = nnz(H); indices of H
+    int64_t *H2i,           // size hnz = nnz(H); indices of H
     Entry *H2x,         // size hnz; numerical values of H
     Entry *H2Tau        // size nh; Householder coefficients
 ) ;
 
-template <typename Entry> Long spqr_rhpack    // returns # of entries in R+H
+template <typename Entry> int64_t spqr_rhpack    // returns # of entries in R+H
 (
     // input, not modified
     int keepH,              // if true, then H is packed
-    Long m,                 // # of rows in F
-    Long n,                 // # of columns in F
-    Long npiv,              // number of pivotal columns in F
-    Long *Stair,            // size npiv; column j is dead if Stair [j] == 0.
+    int64_t m,                 // # of rows in F
+    int64_t n,                 // # of columns in F
+    int64_t npiv,              // number of pivotal columns in F
+    int64_t *Stair,            // size npiv; column j is dead if Stair [j] == 0.
                             // Only the first npiv columns can be dead.
 
     // input, not modified (unless the pack occurs in-place)
@@ -609,7 +607,7 @@ template <typename Entry> Long spqr_rhpack    // returns # of entries in R+H
 
     // output, contents not defined on input
     Entry *R,               // packed columns of R+H
-    Long *p_rm              // number of rows in R block
+    int64_t *p_rm              // number of rows in R block
 ) ;
 
 template <typename Entry> void spqr_hpinv
@@ -619,7 +617,7 @@ template <typename Entry> void spqr_hpinv
     // input/output
     spqr_numeric <Entry> *QRnum,
     // workspace
-    Long *W              // size QRnum->m
+    int64_t *W              // size QRnum->m
 ) ;
 
 template <typename Entry> int spqr_1colamd
@@ -628,17 +626,17 @@ template <typename Entry> int spqr_1colamd
     int ordering,           // all available, except 0:fixed and 3:given
                             // treated as 1:natural
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     cholmod_sparse *A,      // m-by-n sparse matrix
 
     // output arrays, neither allocated nor defined on input.
 
-    Long **p_Q1fill,        // size n+bncols, fill-reducing
+    int64_t **p_Q1fill,        // size n+bncols, fill-reducing
                             // or natural ordering
 
-    Long **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
+    int64_t **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
                             // row of R1.  NULL if n1cols == 0.
-    Long **p_P1inv,         // size m, singleton row inverse permutation.
+    int64_t **p_P1inv,         // size m, singleton row inverse permutation.
                             // If row i of A is the kth singleton row, then
                             // P1inv [i] = k.  NULL if n1cols is zero.
 
@@ -647,8 +645,8 @@ template <typename Entry> int spqr_1colamd
                             // Y = [A B] or Y = [A2 B2].  If B is empty and
                             // there are no column singletons, Y is NULL
 
-    Long *p_n1cols,         // number of column singletons found
-    Long *p_n1rows,         // number of corresponding rows found
+    int64_t *p_n1cols,         // number of column singletons found
+    int64_t *p_n1rows,         // number of corresponding rows found
 
     // workspace and parameters
     cholmod_common *cc
@@ -658,14 +656,14 @@ template <typename Entry> int spqr_1fixed
 (
     // inputs, not modified
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     cholmod_sparse *A,      // m-by-n sparse matrix
 
     // output arrays, neither allocated nor defined on input.
 
-    Long **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
+    int64_t **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
                             // row of R1.  NULL if n1cols == 0.
-    Long **p_P1inv,         // size m, singleton row inverse permutation.
+    int64_t **p_P1inv,         // size m, singleton row inverse permutation.
                             // If row i of A is the kth singleton row, then
                             // P1inv [i] = k.  NULL if n1cols is zero.
 
@@ -674,8 +672,8 @@ template <typename Entry> int spqr_1fixed
                             // Y = [A B] or Y = [A2 B2].  If B is empty and
                             // there are no column singletons, Y is NULL
 
-    Long *p_n1cols,         // number of column singletons found
-    Long *p_n1rows,         // number of corresponding rows found
+    int64_t *p_n1cols,         // number of column singletons found
+    int64_t *p_n1rows,         // number of corresponding rows found
 
     // workspace and parameters
     cholmod_common *cc
@@ -686,47 +684,47 @@ template <typename Entry> SuiteSparseQR_factorization <Entry> *spqr_1factor
     // inputs, not modified
     int ordering,           // all ordering options available
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     int keepH,              // if TRUE, keep the Householder vectors
     cholmod_sparse *A,      // m-by-n sparse matrix
-    Long ldb,               // leading dimension of B, if dense
-    Long *Bp,               // size bncols+1, column pointers of B
-    Long *Bi,               // size bnz = Bp [bncols], row indices of B
+    int64_t ldb,               // leading dimension of B, if dense
+    int64_t *Bp,               // size bncols+1, column pointers of B
+    int64_t *Bi,               // size bnz = Bp [bncols], row indices of B
     Entry *Bx,              // size bnz, numerical values of B
 
     // workspace and parameters
     cholmod_common *cc
 ) ;
 
-Long spqr_cumsum            // returns total sum
+int64_t spqr_cumsum            // returns total sum
 (
     // input, not modified
-    Long n,
+    int64_t n,
 
     // input/output
-    Long *X                 // size n+1. X = cumsum ([0 X])
+    int64_t *X                 // size n+1. X = cumsum ([0 X])
 ) ;
 
 void spqr_shift
 (
     // input, not modified
-    Long n,
+    int64_t n,
 
     // input/output
-    Long *X                 // size n+1
+    int64_t *X                 // size n+1
 ) ;
 
 template <typename Entry> void spqr_larftb
 (
     // inputs, not modified (V is modified and then restored on output)
     int method,     // 0,1,2,3
-    Long m,         // C is m-by-n
-    Long n,
-    Long k,         // V is v-by-k
+    int64_t m,         // C is m-by-n
+    int64_t n,
+    int64_t k,         // V is v-by-k
                     // for methods 0 and 1, v = m,
                     // for methods 2 and 3, v = n
-    Long ldc,       // leading dimension of C
-    Long ldv,       // leading dimension of V
+    int64_t ldc,       // leading dimension of C
+    int64_t ldv,       // leading dimension of V
     Entry *V,       // V is v-by-k, unit lower triangular (diag not stored)
     Entry *Tau,     // size k, the k Householder coefficients
 
@@ -744,18 +742,18 @@ int spqr_happly_work
     // input
     int method,     // 0,1,2,3 
 
-    Long m,         // X is m-by-n
-    Long n,
+    int64_t m,         // X is m-by-n
+    int64_t n,
 
     // FUTURE : make H cholmod_sparse:
-    Long nh,        // number of Householder vectors
-    Long *Hp,       // size nh+1, column pointers for H
-    Long hchunk, 
+    int64_t nh,        // number of Householder vectors
+    int64_t *Hp,       // size nh+1, column pointers for H
+    int64_t hchunk, 
 
     // outputs; sizes of workspaces needed
-    Long *p_vmax, 
-    Long *p_vsize, 
-    Long *p_csize
+    int64_t *p_vmax, 
+    int64_t *p_vsize, 
+    int64_t *p_csize
 ) ;
 
 template <typename Entry> void spqr_happly
@@ -763,12 +761,12 @@ template <typename Entry> void spqr_happly
     // input
     int method,     // 0,1,2,3 
 
-    Long m,         // X is m-by-n
-    Long n,
+    int64_t m,         // X is m-by-n
+    int64_t n,
 
-    Long nh,        // number of Householder vectors
-    Long *Hp,       // size nh+1, column pointers for H
-    Long *Hi,       // size hnz = Hp [nh], row indices of H
+    int64_t nh,        // number of Householder vectors
+    int64_t *Hp,       // size nh+1, column pointers for H
+    int64_t *Hi,       // size hnz = Hp [nh], row indices of H
     Entry *Hx,      // size hnz, Householder values.  Note that the first
                     // entry in each column must be equal to 1.0
 
@@ -778,10 +776,10 @@ template <typename Entry> void spqr_happly
     Entry *X,       // size m-by-n with leading dimension m
 
     // workspace
-    Long vmax,
-    Long hchunk,
-    Long *Wi,       // size vmax
-    Long *Wmap,     // size MAX(mh,1) where H is mh-by-nh
+    int64_t vmax,
+    int64_t hchunk,
+    int64_t *Wi,       // size vmax
+    int64_t *Wmap,     // size MAX(mh,1) where H is mh-by-nh
     Entry *C,       // size csize
     Entry *V,       // size vsize
     cholmod_common *cc
@@ -791,14 +789,14 @@ template <typename Entry> void spqr_panel
 (
     // input
     int method,
-    Long m,
-    Long n,
-    Long v,
-    Long h,             // number of Householder vectors in the panel
-    Long *Vi,           // Vi [0:v-1] defines the pattern of the panel
+    int64_t m,
+    int64_t n,
+    int64_t v,
+    int64_t h,             // number of Householder vectors in the panel
+    int64_t *Vi,           // Vi [0:v-1] defines the pattern of the panel
     Entry *V,           // v-by-h, panel of Householder vectors
     Entry *Tau,         // size h, Householder coefficients for the panel
-    Long ldx,
+    int64_t ldx,
 
     // input/output
     Entry *X,           // m-by-n with leading dimension ldx
@@ -814,29 +812,29 @@ template <typename Entry> int spqr_append       // TRUE if OK, FALSE otherwise
 (
     // inputs, not modified
     Entry *X,       // size m-by-1
-    Long *P,        // size m, or NULL; permutation to apply to X.
+    int64_t *P,        // size m, or NULL; permutation to apply to X.
                     // P [k] = i if row k of A is row i of X
 
     // input/output
     cholmod_sparse *A,    // size m-by-n2 where n2 > n
-    Long *p_n,       // number of columns of A; increased by one
+    int64_t *p_n,       // number of columns of A; increased by one
 
     // workspace and parameters
     cholmod_common *cc
 ) ;
 
-template <typename Entry> Long spqr_trapezoidal       // rank of R, or EMPTY
+template <typename Entry> int64_t spqr_trapezoidal       // rank of R, or EMPTY
 (
     // inputs, not modified
 
-    Long n,         // R is m-by-n (m is not needed here; can be economy R)
-    Long *Rp,       // size n+1, column pointers of R
-    Long *Ri,       // size rnz = Rp [n], row indices of R
+    int64_t n,         // R is m-by-n (m is not needed here; can be economy R)
+    int64_t *Rp,       // size n+1, column pointers of R
+    int64_t *Ri,       // size rnz = Rp [n], row indices of R
     Entry *Rx,      // size rnz, numerical values of R
 
-    Long bncols,    // number of columns of B
+    int64_t bncols,    // number of columns of B
 
-    Long *Qfill,    // size n+bncols, fill-reducing ordering.  Qfill [k] = j if
+    int64_t *Qfill,    // size n+bncols, fill-reducing ordering.  Qfill [k] = j if
                     // the jth column of A is the kth column of R.  If Qfill is
                     // NULL, then it is assumed to be the identity
                     // permutation.
@@ -846,11 +844,11 @@ template <typename Entry> Long spqr_trapezoidal       // rank of R, or EMPTY
                                     // the matrix T is not created.
 
     // outputs, not allocated on input
-    Long **p_Tp,    // size n+1, column pointers of T
-    Long **p_Ti,    // size rnz, row indices of T
+    int64_t **p_Tp,    // size n+1, column pointers of T
+    int64_t **p_Ti,    // size rnz, row indices of T
     Entry **p_Tx,   // size rnz, numerical values of T
 
-    Long **p_Qtrap, // size n+bncols, modified Qfill
+    int64_t **p_Qtrap, // size n+bncols, modified Qfill
 
     // workspace and parameters
     cholmod_common *cc
@@ -864,8 +862,8 @@ template <typename Entry> void spqr_rsolve
     SuiteSparseQR_factorization <Entry> *QR,
     int use_Q1fill,
 
-    Long nrhs,              // number of columns of B
-    Long ldb,               // leading dimension of B
+    int64_t nrhs,              // number of columns of B
+    int64_t ldb,               // leading dimension of B
     Entry *B,               // size m-by-nrhs with leading dimesion ldb
 
     // output
@@ -873,27 +871,27 @@ template <typename Entry> void spqr_rsolve
 
     // workspace
     Entry **Rcolp,
-    Long *Rlive,
+    int64_t *Rlive,
     Entry *W,
 
     cholmod_common *cc
 ) ;
 
 // returns rank of F, or 0 on error
-template <typename Entry> Long spqr_front
+template <typename Entry> int64_t spqr_front
 (
     // input, not modified
-    Long m,             // F is m-by-n with leading dimension m
-    Long n,
-    Long npiv,          // number of pivot columns
+    int64_t m,             // F is m-by-n with leading dimension m
+    int64_t n,
+    int64_t npiv,          // number of pivot columns
     double tol,         // a column is flagged as dead if its norm is <= tol
-    Long ntol,          // apply tol only to first ntol pivot columns
-    Long fchunk,        // block size for compact WY Householder reflections,
+    int64_t ntol,          // apply tol only to first ntol pivot columns
+    int64_t fchunk,        // block size for compact WY Householder reflections,
                         // treated as 1 if fchunk <= 1
 
     // input/output
     Entry *F,           // frontal matrix F of size m-by-n
-    Long *Stair,        // size n, entries F (Stair[k]:m-1, k) are all zero,
+    int64_t *Stair,        // size n, entries F (Stair[k]:m-1, k) are all zero,
                         // and remain zero on output.
     char *Rdead,        // size npiv; all zero on input.  If k is dead,
                         // Rdead [k] is set to 1
@@ -921,7 +919,7 @@ template <typename Entry> int spqr_rmap
 // === spqrgpu features ========================================================
 // =============================================================================
 
-#ifdef GPU_BLAS
+#ifdef SUITESPARSE_CUDA
 #include "spqrgpu.hpp"
 #endif
 
@@ -951,7 +949,7 @@ inline double spqr_abs (double x, cholmod_common *cc)       // cc is unused
 
 inline double spqr_abs (Complex x, cholmod_common *cc)
 {
-    return (SuiteSparse_config.hypot_func (x.real ( ), x.imag ( ))) ;
+    return (SuiteSparse_config_hypot (x.real ( ), x.imag ( ))) ;
 }
 
 
@@ -967,7 +965,7 @@ inline double spqr_divide (double a, double b, cholmod_common *cc)  // cc unused
 inline Complex spqr_divide (Complex a, Complex b, cholmod_common *cc)
 {
     double creal, cimag ;
-    SuiteSparse_config.divcomplex_func
+    SuiteSparse_config_divcomplex
         (a.real(), a.imag(), b.real(), b.imag(), &creal, &cimag) ;
     return (Complex (creal, cimag)) ;
 }
@@ -977,12 +975,12 @@ inline Complex spqr_divide (Complex a, Complex b, cholmod_common *cc)
 // === spqr_add ================================================================
 // =============================================================================
 
-// Add two non-negative Long's, and return the result.  Checks for Long overflow
-// and sets ok to FALSE if it occurs.
+// Add two non-negative int64_t's, and return the result.  Checks for int64_t
+// overflow and sets ok to FALSE if it occurs.
 
-inline Long spqr_add (Long a, Long b, int *ok)
+inline int64_t spqr_add (int64_t a, int64_t b, int *ok)
 {
-    Long c = a + b ;
+    int64_t c = a + b ;
     if (c < 0)
     {
         (*ok) = FALSE ;
@@ -996,12 +994,12 @@ inline Long spqr_add (Long a, Long b, int *ok)
 // === spqr_mult ===============================================================
 // =============================================================================
 
-// Multiply two positive Long's, and return the result.  Checks for Long
+// Multiply two positive int64_t's, and return the result.  Checks for int64_t
 // overflow and sets ok to FALSE if it occurs.
 
-inline Long spqr_mult (Long a, Long b, int *ok)
+inline int64_t spqr_mult (int64_t a, int64_t b, int *ok)
 {
-    Long c = a * b ;
+    int64_t c = a * b ;
     if (((double) c) != ((double) a) * ((double) b))
     {
         (*ok) = FALSE ;
@@ -1010,106 +1008,54 @@ inline Long spqr_mult (Long a, Long b, int *ok)
     return (c) ;
 }
 
+//------------------------------------------------------------------------------
+// test coverage
+//------------------------------------------------------------------------------
 
-// =============================================================================
-// === BLAS interface ==========================================================
-// =============================================================================
+// SuiteSparse_metis has been modified from the original METIS 5.1.0.  It uses
+// the SuiteSparse_config function pointers for malloc/calloc/realloc/free, so
+// that it can use the same memory manager functions as the rest of
+// SuiteSparse.  However, during test coverage in SPQR/Tcov, the call to
+// malloc inside SuiteSparse_metis pretends to fail, to test SPQR's memory
+// handling.  This causes METIS to terminate the program.  To avoid this, METIS
+// is allowed to use the standard ANSI C11 malloc/calloc/realloc/free functions
+// during testing.
 
-// To compile SuiteSparseQR with 64-bit BLAS, use -DBLAS64.  See also
-// CHOLMOD/Include/cholmod_blas.h
+#ifdef TEST_COVERAGE
 
-extern "C" {
-#include "cholmod_blas.h"
-}
+    //--------------------------------------------------------------------------
+    // SPQR during test coverage in SPQR/Tcov.
+    //--------------------------------------------------------------------------
 
-#undef CHECK_BLAS_INT
-#undef EQ
-#define CHECK_BLAS_INT (sizeof (BLAS_INT) < sizeof (Long))
-#define EQ(K,k) (((BLAS_INT) K) == ((Long) k))
+    void normal_memory_handler (cholmod_common *cc, bool free_work) ;
+    void test_memory_handler (cholmod_common *cc, bool free_work) ;
+    extern int64_t my_tries, my_punt, save_my_tries, save_my_punt ;
 
-#ifdef SUN64
+    #define TEST_COVERAGE_PAUSE                                     \
+    {                                                               \
+        save_my_tries = my_tries ;                                  \
+        save_my_punt  = my_punt  ;                                  \
+        normal_memory_handler (cc, false) ;                         \
+    }
 
-#define BLAS_DNRM2    dnrm2_64_
-#define LAPACK_DLARF  dlarf_64_
-#define LAPACK_DLARFG dlarfg_64_
-#define LAPACK_DLARFT dlarft_64_
-#define LAPACK_DLARFB dlarfb_64_
-
-#define BLAS_DZNRM2   dznrm2_64_
-#define LAPACK_ZLARF  zlarf_64_
-#define LAPACK_ZLARFG zlarfg_64_
-#define LAPACK_ZLARFT zlarft_64_
-#define LAPACK_ZLARFB zlarfb_64_
-
-#elif defined (BLAS_NO_UNDERSCORE)
-
-#define BLAS_DNRM2    dnrm2
-#define LAPACK_DLARF  dlarf
-#define LAPACK_DLARFG dlarfg
-#define LAPACK_DLARFT dlarft
-#define LAPACK_DLARFB dlarfb
-
-#define BLAS_DZNRM2   dznrm2
-#define LAPACK_ZLARF  zlarf
-#define LAPACK_ZLARFG zlarfg
-#define LAPACK_ZLARFT zlarft
-#define LAPACK_ZLARFB zlarfb
+    #define TEST_COVERAGE_RESUME                                    \
+    {                                                               \
+        test_memory_handler (cc, false) ;                           \
+        my_tries = save_my_tries ;                                  \
+        my_punt  = save_my_punt  ;                                  \
+    }
 
 #else
 
-#define BLAS_DNRM2    dnrm2_
-#define LAPACK_DLARF  dlarf_
-#define LAPACK_DLARFG dlarfg_
-#define LAPACK_DLARFT dlarft_
-#define LAPACK_DLARFB dlarfb_
+    //--------------------------------------------------------------------------
+    // SPQR in production: no change to SuiteSparse_config
+    //--------------------------------------------------------------------------
 
-#define BLAS_DZNRM2   dznrm2_
-#define LAPACK_ZLARF  zlarf_
-#define LAPACK_ZLARFG zlarfg_
-#define LAPACK_ZLARFT zlarft_
-#define LAPACK_ZLARFB zlarfb_
+
+    #define TEST_COVERAGE_PAUSE
+    #define TEST_COVERAGE_RESUME
 
 #endif
 
-// =============================================================================
-// === BLAS and LAPACK prototypes ==============================================
-// =============================================================================
-
-extern "C"
-{
-
-void LAPACK_DLARFT (char *direct, char *storev, BLAS_INT *n, BLAS_INT *k,
-    double *V, BLAS_INT *ldv, double *Tau, double *T, BLAS_INT *ldt) ;
-
-void LAPACK_ZLARFT (char *direct, char *storev, BLAS_INT *n, BLAS_INT *k,
-    Complex *V, BLAS_INT *ldv, Complex *Tau, Complex *T, BLAS_INT *ldt) ;
-
-void LAPACK_DLARFB (char *side, char *trans, char *direct, char *storev,
-    BLAS_INT *m, BLAS_INT *n, BLAS_INT *k, double *V, BLAS_INT *ldv,
-    double *T, BLAS_INT *ldt, double *C, BLAS_INT *ldc, double *Work,
-    BLAS_INT *ldwork) ;
-
-void LAPACK_ZLARFB (char *side, char *trans, char *direct, char *storev,
-    BLAS_INT *m, BLAS_INT *n, BLAS_INT *k, Complex *V, BLAS_INT *ldv,
-    Complex *T, BLAS_INT *ldt, Complex *C, BLAS_INT *ldc, Complex *Work,
-    BLAS_INT *ldwork) ;
-
-double BLAS_DNRM2 (BLAS_INT *n, double *X, BLAS_INT *incx) ;
-
-double BLAS_DZNRM2 (BLAS_INT *n, Complex *X, BLAS_INT *incx) ;
-
-void LAPACK_DLARFG (BLAS_INT *n, double *alpha, double *X, BLAS_INT *incx,
-    double *tau) ;
-
-void LAPACK_ZLARFG (BLAS_INT *n, Complex *alpha, Complex *X, BLAS_INT *incx,
-    Complex *tau) ;
-
-void LAPACK_DLARF (char *side, BLAS_INT *m, BLAS_INT *n, double *V,
-    BLAS_INT *incv, double *tau, double *C, BLAS_INT *ldc, double *Work) ;
-
-void LAPACK_ZLARF (char *side, BLAS_INT *m, BLAS_INT *n, Complex *V,
-    BLAS_INT *incv, Complex *tau, Complex *C, BLAS_INT *ldc, Complex *Work) ;
-
-}
-
 #endif
+
