@@ -2,6 +2,11 @@
 // === spqr_1colamd ============================================================
 // =============================================================================
 
+// SPQR, Copyright (c) 2008-2022, Timothy A Davis. All Rights Reserved.
+// SPDX-License-Identifier: GPL-2.0+
+
+//------------------------------------------------------------------------------
+
 //  Find column singletons, with column permutations allowed.  After column
 //  singletons are found (and ordered first in Q1fill), the remaining columns
 //  are optionally permuted via COLAMD or CHOLMOD's internal ordering method(s).
@@ -21,17 +26,17 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     int ordering,           // all available, except 0:fixed and 3:given
                             // treated as 1:natural
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     cholmod_sparse *A,      // m-by-n sparse matrix
 
     // outputs, neither allocated nor defined on input
 
-    Long **p_Q1fill,        // size n+bncols, fill-reducing
+    int64_t **p_Q1fill,        // size n+bncols, fill-reducing
                             // or natural ordering
 
-    Long **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
+    int64_t **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
                             // row of R1.  NULL if n1cols == 0.
-    Long **p_P1inv,         // size m, singleton row inverse permutation.
+    int64_t **p_P1inv,         // size m, singleton row inverse permutation.
                             // If row i of A is the kth singleton row, then
                             // P1inv [i] = k.  NULL if n1cols is zero.
 
@@ -40,17 +45,17 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
                             // Y = [A B] or Y = [A2 B2].  If B is empty and
                             // there are no column singletons, Y is NULL
 
-    Long *p_n1cols,         // number of column singletons found
-    Long *p_n1rows,         // number of corresponding rows found
+    int64_t *p_n1cols,         // number of column singletons found
+    int64_t *p_n1rows,         // number of corresponding rows found
 
     // workspace and parameters
     cholmod_common *cc
 )
 {
-    Long *Q1fill, *Degree, *Qrows, *W, *Winv, *ATp, *ATj, *R1p, *P1inv, *Yp,
+    int64_t *Q1fill, *Degree, *Qrows, *W, *Winv, *ATp, *ATj, *R1p, *P1inv, *Yp,
         *Ap, *Ai, *Work ;
     Entry *Ax ;
-    Long p, d, j, i, k, n1cols, n1rows, row, pend, n2rows, n2cols = EMPTY,
+    int64_t p, d, j, i, k, n1cols, n1rows, row, pend, n2rows, n2cols = EMPTY,
         nz2, kk, p2, col2, ynz, fill_reducing_ordering, m, n, xtype, worksize ;
     cholmod_sparse *AT, *Y ;
 
@@ -62,8 +67,8 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
 
     m = A->nrow ;
     n = A->ncol ;
-    Ap = (Long *) A->p ;
-    Ai = (Long *) A->i ;
+    Ap = (int64_t *) A->p ;
+    Ai = (int64_t *) A->i ;
     Ax = (Entry *) A->x ;
 
     // set outputs to NULL in case of early return
@@ -78,7 +83,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     // allocate result Q1fill (Y, R1p, P1inv allocated later)
     // -------------------------------------------------------------------------
 
-    Q1fill = (Long *) cholmod_l_malloc (n+bncols, sizeof (Long), cc) ;
+    Q1fill = (int64_t *) cholmod_l_malloc (n+bncols, sizeof (int64_t), cc) ;
 
     // -------------------------------------------------------------------------
     // allocate workspace
@@ -91,7 +96,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
 
     worksize = ((fill_reducing_ordering) ? 3:2) * n ;
 
-    Work = (Long *) cholmod_l_malloc (worksize, sizeof (Long), cc) ;
+    Work = (int64_t *) cholmod_l_malloc (worksize, sizeof (int64_t), cc) ;
     Degree = Work ;         // size n
     Qrows  = Work + n ;     // size n
     Winv   = Qrows ;        // Winv and Qrows not needed at the same time
@@ -100,8 +105,8 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     if (cc->status < CHOLMOD_OK)
     {
         // out of memory; free everything and return
-        cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
-        cholmod_l_free (n+bncols, sizeof (Long), Q1fill, cc) ;
+        cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
+        cholmod_l_free (n+bncols, sizeof (int64_t), Q1fill, cc) ;
         return (FALSE) ;
     }
 
@@ -153,13 +158,13 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     if (cc->status < CHOLMOD_OK)
     {
         // out of memory; free everything and return
-        cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
-        cholmod_l_free (n+bncols, sizeof (Long), Q1fill, cc) ;
+        cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
+        cholmod_l_free (n+bncols, sizeof (int64_t), Q1fill, cc) ;
         return (FALSE) ;
     }
 
-    ATp = (Long *) AT->p ;
-    ATj = (Long *) AT->i ;
+    ATp = (int64_t *) AT->p ;
+    ATj = (int64_t *) AT->i ;
 
     // -------------------------------------------------------------------------
     // remove column singletons via breadth-first-search
@@ -172,7 +177,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
         // get a new singleton from the queue
         // ---------------------------------------------------------------------
 
-        // Long col = Q1fill [k] ;   unused variable, for debugging
+        // int64_t col = Q1fill [k] ;   unused variable, for debugging
         #define col (Q1fill [k])
 
         row = Qrows [k] ;
@@ -271,7 +276,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     ASSERT (k == n - n1cols) ;
     for (k = 0 ; k < n1cols ; k++)
     {
-        Long col = Q1fill [k] ;
+        int64_t col = Q1fill [k] ;
         ASSERT (Degree [col] <= 0) ;
     }
     k = 0 ;
@@ -311,17 +316,17 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
         // ---------------------------------------------------------------------
 
         // allocate result arrays R1p and P1inv
-        R1p   = (Long *) cholmod_l_malloc (n1rows+1, sizeof (Long), cc) ;
-        P1inv = (Long *) cholmod_l_malloc (m,        sizeof (Long), cc) ;
+        R1p   = (int64_t *) cholmod_l_malloc (n1rows+1, sizeof (int64_t), cc) ;
+        P1inv = (int64_t *) cholmod_l_malloc (m,        sizeof (int64_t), cc) ;
 
         if (cc->status < CHOLMOD_OK)
         {
             // out of memory; free everything and return
             cholmod_l_free_sparse (&AT, cc) ;
-            cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
-            cholmod_l_free (n+bncols, sizeof (Long), Q1fill, cc) ;
-            cholmod_l_free (n1rows+1, sizeof (Long), R1p, cc) ;
-            cholmod_l_free (m,        sizeof (Long), P1inv, cc) ;
+            cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
+            cholmod_l_free (n+bncols, sizeof (int64_t), Q1fill, cc) ;
+            cholmod_l_free (n1rows+1, sizeof (int64_t), R1p, cc) ;
+            cholmod_l_free (m,        sizeof (int64_t), P1inv, cc) ;
             return (FALSE) ;
         }
 
@@ -481,14 +486,14 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
 
         PR (("n1cols %ld n1rows %ld n2cols %ld n2rows %ld\n",
             n1cols, n1rows, n2cols, n2rows)) ;
-        ASSERT ((Long) AT->nrow == n) ;
-        ASSERT ((Long) AT->ncol == m) ;
+        ASSERT ((int64_t) AT->nrow == n) ;
+        ASSERT ((int64_t) AT->ncol == m) ;
 
         AT->nrow = n2cols ;
         AT->ncol = n2rows ;
 
         // save the current CHOLMOD settings
-        Long save [6] ;
+        int64_t save [6] ;
         save [0] = cc->supernodal ;
         save [1] = cc->nmethods ;
         save [2] = cc->postorder ;
@@ -503,13 +508,10 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
         if (ordering == SPQR_ORDERING_BEST)
         {
             ordering = SPQR_ORDERING_CHOLMOD ;
-            cc->nmethods = 2 ;
+            cc->nmethods = 3 ;
             cc->method [0].ordering = CHOLMOD_COLAMD ;
             cc->method [1].ordering = CHOLMOD_AMD ;
-#ifndef NPARTITION
-            cc->nmethods = 3 ;
             cc->method [2].ordering = CHOLMOD_METIS ;
-#endif
         }
 
         // 9:bestamd: best of COLAMD(A) and AMD(A'A)
@@ -522,55 +524,25 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
             cc->method [1].ordering = CHOLMOD_AMD ;
         }
 
-#ifdef NPARTITION
-        if (ordering == SPQR_ORDERING_METIS)
-        {
-            // METIS not installed; use default ordering
-            ordering = SPQR_ORDERING_DEFAULT ;
-        }
-#endif
-
         if (ordering == SPQR_ORDERING_DEFAULT)
         {
             // Version 1.2.0:  just use COLAMD
             ordering = SPQR_ORDERING_COLAMD ;
-
-#if 0
-            // Version 1.1.2 and earlier:
-            if (n2rows <= 2*n2cols)
-            {
-                // just use COLAMD; do not try AMD or METIS
-                ordering = SPQR_ORDERING_COLAMD ;
-            }
-            else
-            {
-#ifndef NPARTITION
-                // use CHOLMOD's default ordering: try AMD and then METIS
-                // if AMD gives high fill-in, and take the best ordering found
-                ordering = SPQR_ORDERING_CHOLMOD ;
-                cc->nmethods = 0 ;
-#else
-                // METIS is not installed, so just use AMD
-                ordering = SPQR_ORDERING_AMD ;
-#endif
-            }
-#endif
-
         }
 
         if (ordering == SPQR_ORDERING_AMD)
         {
             // use CHOLMOD's interface to AMD to order A'*A
-            cholmod_l_amd (AT, NULL, 0, (Long *) (Q1fill + n1cols), cc) ;
+            cholmod_l_amd (AT, NULL, 0, (int64_t *) (Q1fill + n1cols), cc) ;
         }
-#ifndef NPARTITION
         else if (ordering == SPQR_ORDERING_METIS)
         {
             // use CHOLMOD's interface to METIS to order A'*A (if installed)
+            TEST_COVERAGE_PAUSE ;
             cholmod_l_metis (AT, NULL, 0, TRUE,
-                (Long *) (Q1fill + n1cols), cc) ;
+                (int64_t *) (Q1fill + n1cols), cc) ;
+            TEST_COVERAGE_RESUME ;
         }
-#endif
         else if (ordering == SPQR_ORDERING_CHOLMOD)
         {
             // use CHOLMOD's internal ordering (defined by cc) to order AT
@@ -578,11 +550,13 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
             cc->supernodal = CHOLMOD_SIMPLICIAL ;
             cc->postorder = TRUE ;
             cholmod_factor *Sc ;
+            TEST_COVERAGE_PAUSE ;
             Sc = cholmod_l_analyze_p2 (FALSE, AT, NULL, NULL, 0, cc) ;
+            TEST_COVERAGE_RESUME ;
             if (Sc != NULL)
             {
                 // copy perm from Sc->Perm [0:n2cols-1] to Q1fill (n1cols:n)
-                Long *Sc_perm = (Long *) Sc->Perm ;
+                int64_t *Sc_perm = (int64_t *) Sc->Perm ;
                 for (k = 0 ; k < n2cols ; k++)
                 {
                     Q1fill [k + n1cols] = Sc_perm [k] ;
@@ -604,7 +578,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
             // use CHOLMOD's interface to COLAMD to order AT
             ordering = SPQR_ORDERING_COLAMD ;
             cholmod_l_colamd (AT, NULL, 0, TRUE,
-                (Long *) (Q1fill + n1cols), cc) ;
+                (int64_t *) (Q1fill + n1cols), cc) ;
         }
 
         cc->SPQR_istat [7] = ordering ;
@@ -634,10 +608,10 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     if (cc->status < CHOLMOD_OK)
     {
         // out of memory; free everything and return
-        cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
-        cholmod_l_free (n+bncols, sizeof (Long), Q1fill, cc) ;
-        cholmod_l_free (n1rows+1, sizeof (Long), R1p, cc) ;
-        cholmod_l_free (m,        sizeof (Long), P1inv, cc) ;
+        cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
+        cholmod_l_free (n+bncols, sizeof (int64_t), Q1fill, cc) ;
+        cholmod_l_free (n1rows+1, sizeof (int64_t), R1p, cc) ;
+        cholmod_l_free (m,        sizeof (int64_t), P1inv, cc) ;
         return (FALSE) ;
     }
 
@@ -704,14 +678,14 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
         if (cc->status < CHOLMOD_OK)
         {
             // out of memory; free everything and return
-            cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
-            cholmod_l_free (n+bncols, sizeof (Long), Q1fill, cc) ;
-            cholmod_l_free (n1rows+1, sizeof (Long), R1p, cc) ;
-            cholmod_l_free (m,        sizeof (Long), P1inv, cc) ;
+            cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
+            cholmod_l_free (n+bncols, sizeof (int64_t), Q1fill, cc) ;
+            cholmod_l_free (n1rows+1, sizeof (int64_t), R1p, cc) ;
+            cholmod_l_free (m,        sizeof (int64_t), P1inv, cc) ;
             return (FALSE) ;
         }
 
-        Yp = (Long *) Y->p ; 
+        Yp = (int64_t *) Y->p ; 
 
         ynz = 0 ;
         PR (("1c wrapup: n1cols %ld n %ld\n", n1cols, n)) ;
@@ -730,7 +704,7 @@ template <typename Entry> int spqr_1colamd  // TRUE if OK, FALSE otherwise
     // free workspace and return results
     // -------------------------------------------------------------------------
 
-    cholmod_l_free (worksize, sizeof (Long), Work, cc) ;
+    cholmod_l_free (worksize, sizeof (int64_t), Work, cc) ;
 
     *p_Q1fill = Q1fill ;
     *p_R1p    = R1p ;
@@ -750,17 +724,17 @@ template int spqr_1colamd <double>
     int ordering,           // all available, except 0:fixed and 3:given
                             // treated as 1:natural
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     cholmod_sparse *A,      // m-by-n sparse matrix
 
     // output arrays, neither allocated nor defined on input.
 
-    Long **p_Q1fill,        // size n+bncols, fill-reducing
+    int64_t **p_Q1fill,        // size n+bncols, fill-reducing
                             // or natural ordering
 
-    Long **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
+    int64_t **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
                             // row of R1.  NULL if n1cols == 0.
-    Long **p_P1inv,         // size m, singleton row inverse permutation.
+    int64_t **p_P1inv,         // size m, singleton row inverse permutation.
                             // If row i of A is the kth singleton row, then
                             // P1inv [i] = k.  NULL if n1cols is zero.
 
@@ -769,8 +743,8 @@ template int spqr_1colamd <double>
                             // Y = [A B] or Y = [A2 B2].  If B is empty and
                             // there are no column singletons, Y is NULL
 
-    Long *p_n1cols,         // number of column singletons found
-    Long *p_n1rows,         // number of corresponding rows found
+    int64_t *p_n1cols,         // number of column singletons found
+    int64_t *p_n1rows,         // number of corresponding rows found
 
     // workspace and parameters
     cholmod_common *cc
@@ -784,17 +758,17 @@ template int spqr_1colamd <Complex>
     int ordering,           // all available, except 0:fixed and 3:given
                             // treated as 1:natural
     double tol,             // only accept singletons above tol
-    Long bncols,            // number of columns of B
+    int64_t bncols,            // number of columns of B
     cholmod_sparse *A,      // m-by-n sparse matrix
 
     // output arrays, neither allocated nor defined on input.
 
-    Long **p_Q1fill,        // size n+bncols, fill-reducing
+    int64_t **p_Q1fill,        // size n+bncols, fill-reducing
                             // or natural ordering
 
-    Long **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
+    int64_t **p_R1p,           // size n1rows+1, R1p [k] = # of nonzeros in kth
                             // row of R1.  NULL if n1cols == 0.
-    Long **p_P1inv,         // size m, singleton row inverse permutation.
+    int64_t **p_P1inv,         // size m, singleton row inverse permutation.
                             // If row i of A is the kth singleton row, then
                             // P1inv [i] = k.  NULL if n1cols is zero.
 
@@ -803,8 +777,8 @@ template int spqr_1colamd <Complex>
                             // Y = [A B] or Y = [A2 B2].  If B is empty and
                             // there are no column singletons, Y is NULL
 
-    Long *p_n1cols,         // number of column singletons found
-    Long *p_n1rows,         // number of corresponding rows found
+    int64_t *p_n1cols,         // number of column singletons found
+    int64_t *p_n1rows,         // number of corresponding rows found
 
     // workspace and parameters
     cholmod_common *cc
